"use strict";

import * as browser from "webextension-polyfill";
import * as EWE from "@eyeo/webext-sdk";
import { storageLocalGet, testStorage, isDataCorrupted } from "./storage.js";
import { detectFirstRun } from "./first-run.js";
import { Blocker, PAUSED_KEY } from "./blocker.js";
import { Telemetry } from "./telemetry.js";
import { Subscriptions } from "./subscriptions.js";
import isolatedV3Code from "./snippets/mv3.isolated.mjs";
import injectedV3Code from "./snippets/mv3.injected.mjs";
import { setValidatedMethodContext } from "../lib/validated-method.js";
import { Guardian } from "./guardian.js";
import { subscriptionsMeta } from "./subscriptions-meta.js";

// -----------------------------------------------------------------------
// Configuration
// -----------------------------------------------------------------------
const ADDON_NAME = "uBlock";
const ADDON_VERSION = "23.1.0";

// -----------------------------------------------------------------------
// Pre-start
// -----------------------------------------------------------------------

const blocker = new Blocker(ADDON_NAME, ADDON_VERSION);
const telemetry = new Telemetry(ADDON_NAME, ADDON_VERSION);
const subscriptions = new Subscriptions();
const guardian = new Guardian();

// Load up context for a real extension. This is overhead so that the front end can also run without the extension
setValidatedMethodContext({ blocker, telemetry, subscriptions, guardian });

// Load message handlers
import { setValidatedMethodBrowser } from "../lib/validated-method.js";
const browserMethods = {
  sendMessage: browser.runtime.sendMessage,
  addListener: browser.runtime.onMessage.addListener,
  getURL: browser.runtime.getURL,
  browserTabs: browser.tabs,
};
setValidatedMethodBrowser(browserMethods);
import "../api/api.js";
import "../api/qa.js";

// -----------------------------------------------------------------------
// Startup
// -----------------------------------------------------------------------

EWE.snippets.setLibrary({
  isolatedCode: isolatedV3Code,
  injectedCode: injectedV3Code,
});

const startTask = EWE.start({
  name: ADDON_NAME,
  version: ADDON_VERSION,
  bundledSubscriptions: subscriptionsMeta.subscriptions,
  bundledSubscriptionsPath: "rules",
});

const tasks = [startTask];

tasks.push(
  testStorage().catch(() => {
    isDataCorrupted(true);
  })
);

Promise.all(tasks).then((results) => {
  const eweFirstRun = results[0];
  finalize(eweFirstRun).then(() => {
    guardian.setStatus("initialized", true);
  });
});

async function finalize(eweFirstRun) {
  if (eweFirstRun) {
    await detectFirstRun(
      eweFirstRun.foundSubscriptions,
      eweFirstRun.foundStorage
    );
  }

  // Add default filter lists
  await blocker.addSubscriptions();
  // Clean up any "extra" filter lists
  await blocker.removeSubscriptions();
  // Start telemetry
  await telemetry.scheduleStatsEvent();
  // Start subscription sync
  await subscriptions.scheduleSyncEvent();
  // Remove AA
  // await EWE.filters.disable([EWE.filters.ACCEPTABLE_ADS_URL]);
  // await EWE.filters.disable([EWE.filters.ACCEPTABLE_ADS_PRIVACY_URL]);

  // If uBlock was paused on shutdown (ublock_is_paused is true), then
  // unpause / remove the allow-list all entry at startup.
  const response = await storageLocalGet(PAUSED_KEY);
  if (response[PAUSED_KEY]) {
    // Unpause after a brief delay to let things settle
    setTimeout(() => blocker.isPaused(false), 200);
  }
}
