// Survey tracks information about surveys to be shown to users.
export class Survey {
  constructor() {
    // To prevent bugs in the extension or browser, we only show one survey
    // per browser start up.
    this.allowed = true;
  }

  /**
   * handle the response from a ping server showing a survey if appropriate.
   *
   * @param data {Object} the survey options data
   */
  handle(data) {
    const options = this.surveyOptionsFrom(data);
    if (options == null) {
      return Promise.resolve();
    }
    switch (options.event) {
      case 1: // show tab "best effort"
        this.showTab(options);
        break;
      default:
      // Do something with an unsupported event?
    }
  }

  /**
   * validUrl checks if the given URL is an allowed survey target.
   *
   * @param url {string} the URL to check
   * @return {boolean} true if the URL is valid, false otherwise
   */
  validUrl(url) {
    return url.startsWith("https://ublock.org/");
  }

  /**
   * surveyOptionsFrom parses the response from a ping to see if it contains valid survey instructions.
   * If so, return an object containing data about the survey to show.
   * Otherwise, return null.
   *
   * @param options {Object} the survey options from the ping server
   * @return {Object|null} valid survey options or null if there was no valid survey in the response
   */
  surveyOptionsFrom(options) {
    // Is there any validation we need to do on the options at this point?
    if (!options.hasOwnProperty("event") || !options.hasOwnProperty("id")) {
      return null;
    }
    switch (options.event) {
      case 0:
        if (!options.hasOwnProperty("settings")) {
          return null;
        }
        break;
      case 1:
        if (!options.hasOwnProperty("url")) {
          return null;
        }
        if (!this.validUrl(options.url)) {
          return null;
        }
        break;
      case 2:
        if (!options.hasOwnProperty("url")) {
          return null;
        }
        if (!this.validUrl(options.url)) {
          return null;
        }
        break;
      case 3:
        if (!options.hasOwnProperty("url")) {
          return null;
        }
        if (!this.validUrl(options.url)) {
          return null;
        }
        break;
      default:
        return null;
    }
    return options;
  }

  /**
   * shouldShowSurvey calls the callback if the survey should be shown.
   * This can be much more extensive a check in the future but for now, we just
   * make sure that this is the one and only survey for this browser start.
   *
   * @param options {Object} information about the survey
   * @param callback {Function} function to call if the survey should be performed
   */
  shouldShowSurvey(options, callback) {
    if (!this.allowed) {
      return;
    }
    // We check if we have already displayed this survey. If we have, don't
    // show it again.
    const key = "surveys." + options.id;
    browser.storage.local.get(key).then((surveys) => {
      if (!surveys.hasOwnProperty(key)) {
        // set this survey
        browser.storage.local
          .set({ [key]: { shown: new Date().toISOString() } })
          .then(() => {
            this.allowed = true;
            callback();
          });
      }
    });
  }

  /**
   * showTab displays a new tab based on survey options.
   *
   * @param options {Object} the survey options for the tab opening
   */
  showTab(options) {
    const openTabIfAllowed = () => {
      setTimeout(() => {
        this.shouldShowSurvey(options, (responseData) => {
          browser.tabs
            .create({ url: `https://ublock.org/${responseData.url}` })
            .then();
        });
      }, 10000); // 10 seconds
    };

    const waitForUserAction = () => {
      browser.tabs.onCreated.removeListener(waitForUserAction);
      openTabIfAllowed();
    };

    browser.idle.queryState(60).then((state) => {
      if (state === "active") {
        openTabIfAllowed();
      } else {
        browser.tabs.onCreated.removeListener(waitForUserAction);
        browser.tabs.onCreated.addListener(waitForUserAction);
      }
    });
  }
}
